# dpro 插件开发

`dpro` 开放了自定义开发插件能力，让开发者 `1s` 钟开发出自己的插件，提高生产效率，成为 钉钉闪电侠⚡️。开发者可以一起完善整个工具的插件生态。<br />由于需要支持 `dpro` 的开发链路调试，初始化插件脚手架的时候系统默认会以 `-D` 的模式，在插件目录运行安装 `dingtalk-cli-pro` 包，但发布插件的 `package.json` 没有依赖。


## 开发插件
```dart
# 安装工具
$ yarn global add dingtalk-cli-pro # 或者 tnpm install -g dingtalk-cli-pro
# 安装插件
$ dpro g  ## 可交互命令行，按步骤操作
# 开发插件
$ vi index.js
```


### 使用案例
![](https://intranetproxy.alipay.com/skylark/lark/0/2019/gif/40822/1565585291000-d1a9649a-21ba-465f-b7fd-404493c7feef.gif#align=left&display=inline&height=442&originHeight=442&originWidth=654&size=0&status=done&width=654)


### 插件目录
`index.js` 为插件主入口， `dingtalkrc.js` 为调试配置文件，可以通过此配置文件，加载当前插件。
```javascript
├── LICENSE.md
├── babel.config.js
├── dingtalkrc.js
├── index.js
├── jest.config.js
├── package.json
├── readme.md
└── test
    └── index.test.js
```


### 代码案例
对于多个异步操作，插件开发同时支持 `async await`  方式，只需要把 `action(() => {})` 改写成 `action(async () => {})` 即可。

```javascript
module.exports = function(dingtalk) {
  /**
  * 命令行工具
  * @see https://www.npmjs.com/package/commander
  */
 // 可以使用 dingtalk or this 作用域
 dingtalk
  // 注册插件命令 $ dpro helloworld
  .registryCommand('helloworld') 
  // 插件命令描述
  .description('welcome to dpro')
  // 运行命令回调
  .action(() => {
    this.log.warn(`Hi ${this.getGitUserInfo().username}, Welcome to dpro`);
  });
}
```


## 脚手架插件开发

### 生命周期
```
configuring -> writing -> install -> end
```

- configuring: 配置基础信息，配置模板地址(origin), 生成脚手架模板地址(target)
- writing: 拷贝文件以及模板文件
- install: 安装依赖
- end: 结束脚手架，可以使用 `spawnCommand` 脚本开启项目工程

## 调试插件
通过 `dingtalkrc.js` 文件配置当前插件入口文件，数组形式可以加载多个插件，配置如下
```javascript
module.exports = {
  "plugins": [
    [
      "./index.js"
    ]
  ]
}
```


### 运行命令调试
```dart
$ dpro command # 命令
```


### vscode调试
插件的开发调试支持VSCode调试， `dpro g ` 命令生成脚手架的时候预制了 `Plugin Param Test` 和 `Plugin Jest Test` 两种命令，可以运行 Vscode 的 debug 模式查看运行。

- Plugin Param Test： 可以在Vscode 的 Debug 模式下设置断点
- Plugin Jest Test： 可以跑 Jest 的测试用例 


## 发布插件
发布插件需要删除或者更改 `dingtalkrc.js` 文件，例如可以更改文件为 `dingtalk.prod.js` ， 通过 `tnpm` 或者 `npm` 方式发布。
```dart
# 阿里内部插件
$ tnpm publish
# 公网使用插件
$ npm publish
```


## API 接口

### 1. 文件模块
`dintalk.fs|this.fs`上下文对象, [Mem-fs-editor API参考文档](https://www.npmjs.com/package/mem-fs-editor)

### 2. 命令行模块
`dingtalk.registryCommand|this.registryCommand`上下文对象, [Commander API 参考文档](https://www.npmjs.com/package/commander)

### 3. 可交互模块
`dingtalk.prompt|this.prompt`上下文对象, [Inquirer API 参考文档](https://www.npmjs.com/package/inquirer)

### 4. 日志模块
`dingtalk.log|this.log`上下文对象， [Log API 参考文档](https://yuque.antfin-inc.com/dingcontent/on7glz/ngtz5w/lib/log.js)

## 

## 常用插件列表

### 1. 静态服务器
```
$ tnpm i dingtalk-plugin-serve -g
```

